import pydeck as pdk

H3_VERSION = "~3.7.*"
DECKGL_VERSION = "~8.8.*"

LIBRARIES_TO_INCLUDE = [
    f"npm/h3-js@{H3_VERSION}/dist/h3-js.umd.js",
    f"npm/@deck.gl/extensions@{DECKGL_VERSION}/dist.min.js",
    f"npm/@deck.gl/carto@{DECKGL_VERSION}/dist.min.js",
]
SELECTED_LIBRARIES = ",".join(LIBRARIES_TO_INCLUDE)
CARTO_LAYER_BUNDLE_URL = f"https://cdn.jsdelivr.net/combine/{SELECTED_LIBRARIES}"


class MapType:
    QUERY = pdk.types.String("query")
    TABLE = pdk.types.String("table")
    TILESET = pdk.types.String("tileset")


class CartoConnection:
    CARTO_DW = pdk.types.String("carto_dw")


class GeoColumnType:
    H3 = pdk.types.String("h3")
    QUADBIN = pdk.types.String("quadbin")


def register_carto_layer():
    """Add CartoLayer JS bundle to pydeck's custom libraries."""
    library_name = "CartoLayerLibrary"
    custom_library = {
        "libraryName": library_name,
        "resourceUri": CARTO_LAYER_BUNDLE_URL,
    }
    default_layer_attributes = {
        "CartoLayer": {
            "client_id": pdk.types.String("pydeck-carto"),
            "on_data_error": pdk.types.Function("notifyError"),
        }
    }
    configuration = """{
        functions: {
            notifyError: () => {
                return (e) => {
                    document.body.style.cssText = 'padding:24px;font-size:14px;font-family:monospace';
                    document.body.innerHTML = `<b>Layer Error</b>: ${e.message}`;
                }
            }
        }
    }"""  # noqa

    if pdk.settings.configuration is None:
        pdk.settings.configuration = configuration

    if pdk.settings.default_layer_attributes is None:
        pdk.settings.default_layer_attributes = default_layer_attributes
    else:
        pdk.settings.default_layer_attributes = {
            **pdk.settings.default_layer_attributes,
            **default_layer_attributes,
        }

    if pdk.settings.custom_libraries is None:
        pdk.settings.custom_libraries = [custom_library]
    else:
        exists = any(
            [
                x.get("libraryName") == library_name
                for x in pdk.settings.custom_libraries
            ]
        )
        if not exists:
            pdk.settings.custom_libraries.append(custom_library)


def get_layer_credentials(carto_auth) -> dict:
    """Get the layer credentials object to gather information
    from carto warehouses.

    The return object has the following structure:
    ``{"apiVersion": "v3", "apiBaseUrl": "...", "accessToken": "...",}``
    """
    api_base_url = carto_auth.get_api_base_url()
    access_token = carto_auth.get_access_token()
    return {
        "apiVersion": "v3",
        "apiBaseUrl": api_base_url,
        "accessToken": access_token,
    }



import pydeck as pdk

from typing import Union


def color_bins(
    attr: str,
    domain: list,
    colors: Union[str, list] = "PurpOr",
    null_color: list = [204, 204, 204],
):
    """Helper function for quickly creating a color bins style.

    Data values of each attribute are rounded down to the nearest value
    in the domain and are then styled with the corresponding color.

    Parameters
    ----------
    attr : str
        Attribute or column to symbolize by.
    domain : list
        Assign manual class break values.
    colors : Union[str, list], optional
        Color assigned to each domain value.
        - str: A valid named CARTOColors palette.
        - list: Array of colors in RGBA ``[ [r, g, b, [a]] ]``.
        Default is PurpOr.
    null_color : list, optional
        Color for null values.
        Default is [204, 204, 204].
    """
    return pdk.types.Function(
        "colorBins",
        **{"attr": attr, "domain": domain, "colors": colors, "nullColor": null_color}
    )


def color_categories(
    attr: str,
    domain: list,
    colors: Union[str, list] = "PurpOr",
    null_color: list = [204, 204, 204],
    others_color: list = [119, 119, 119],
):
    """Helper function for quickly creating a color category style.

    Data values of each attribute listed in the domain are mapped one to one
    with corresponding colors in the range.

    Parameters
    ----------
    attr : str
        Attribute or column to symbolize by.
    domain : list
        Category list. Must be a valid list of categories.
    colors : Union[str, list], optional
        Color assigned to each domain value.
        - str: A valid named CARTOColors palette.
        - list: Array of colors in RGBA ``[ [r, g, b, [a]] ]``.
        Default: PurpOr.
    null_color : list, optional
        Color for null values.
        Default is [204, 204, 204].
    others_color : list, optional
        Fallback color for a category not correctly assigned.
        Default is [119, 119, 119].
    """
    return pdk.types.Function(
        "colorCategories",
        **{
            "attr": attr,
            "domain": domain,
            "colors": colors,
            "nullColor": null_color,
            "othersColor": others_color,
        }
    )


def color_continuous(
    attr: str,
    domain: list,
    colors: Union[str, list] = "PurpOr",
    null_color: list = [204, 204, 204],
):
    """Helper function for quickly creating a color continuous style.

    Data values of each field are interpolated linearly across values in the domain
    and are then styled with a blend of the corresponding color in the range.

    Parameters
    ----------
    attr : str
        Attribute or column to symbolize by.
    domain : list
        Attribute domain to define the data range.
    colors : Union[str, list], optional
        Color assigned to each domain value.
        - str: A valid named CARTOColors palette.
        - list: Array of colors in RGBA ``[ [r, g, b, [a]] ]``.
        Default is PurpOr.
    null_color : list, optional
        Color for null values.
        Default is [204, 204, 204].
    """
    return pdk.types.Function(
        "colorContinuous",
        **{"attr": attr, "domain": domain, "colors": colors, "nullColor": null_color}
    )



from os.path import join
from setuptools import find_packages, setup

version_ns = {}
with open(join("pydeck_carto", "_version.py")) as f:
    exec(f.read(), {}, version_ns)

setup(
    name="pydeck-carto",
    version=version_ns["__version__"],
    description="Pydeck wrapper for use with CARTO",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    keywords=["pydeck", "carto", "visualization", "graphics", "GIS", "maps"],
    author="CARTO",
    author_email="jarroyo@carto.com",
    url="https://github.com/visgl/deck.gl/tree/master/bindings/pydeck-carto",
    license="BSD 3-Clause",
    packages=find_packages(exclude=["examples", "tests"]),
    python_requires=">=3.7",
    install_requires=[
        "pydeck>=0.8.0",
        "carto-auth>=0.1.0",
    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Jupyter",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    zip_safe=False,
)



from ._version import __version__
from .layer import (
    MapType,
    CartoConnection,
    GeoColumnType,
    register_carto_layer,
    get_layer_credentials,
)
from . import styles

__all__ = [
    "__version__",
    "MapType",
    "CartoConnection",
    "GeoColumnType",
    "register_carto_layer",
    "get_layer_credentials",
    "styles",
]
